@@ -6,13 +6,13 @@ from .common import (
 )
 
 from ..errors import (
-    TwitchError,
+    SiteError,
     NoChatReplay,
     VideoUnavailable,
     UserNotFound
 )
 
-from ..utils import (
+from ..utils.core import (
     ensure_seconds,
     timestamp_to_microseconds,
     seconds_to_time,
@@ -36,6 +36,11 @@ from requests.exceptions import RequestException
 from json.decoder import JSONDecodeError
 
 
+class TwitchError(SiteError):
+    """Raised when an error occurs with a Twitch video."""
+    pass
+
+
 # TODO export as another module?
 
 
